module CliUtils ( printMyUUID
                , requestFrom
                , requestFrom'
                , encodeGlobalState
                , showGlobalState
                , getPeerUUID
                , getPeer
                )
where

import RIO
import qualified RIO.ByteString as B
import qualified RIO.ByteString.Lazy as LB
import RIO.List (headMaybe)
import qualified RIO.Text as T
import qualified RIO.HashMap as H

import Control.Monad.Trans.Maybe
import Data
import Data.Binary (encode)
import Data.UUID
import IOData
import IOClasses
import Peer

printSomethingOfConfig :: (HasMailSyncConfig env)
                       =>  (GlobalState -> String)
                       -> RIO env ()
printSomethingOfConfig f = slurpGlobalState >>=
  B.putStr . encodeUtf8 . T.pack . (++ "\n") . f 

printMyUUID :: (HasMailSyncConfig env) => RIO env ()
printMyUUID = printSomethingOfConfig $ show . myUUID

showGlobalState :: (HasMailSyncConfig env) => RIO env ()
showGlobalState = printSomethingOfConfig $ show

peerSince :: UUID -> GlobalState -> Integer
peerSince peer gs =
  maybe 0 ((+ 1) . fromId)
  . H.lookup peer
  $ upstreamPeers gs

getPeer :: (HasMailSyncConfig env) => String -> RIO env (Maybe Peer)
getPeer ident = do
  peers <- view mailSyncPeersL
  let filterPeer = filter ((== ident) . pName) peers
  return $ headMaybe filterPeer


getPeerUUID :: (HasMailSyncConfig env) => String -> RIO env (Maybe UUID)
getPeerUUID ident = do
  let mUUID = readMaybe ident 
  peers <- view mailSyncPeersL
  let filterPeer = filter ((== ident) . pName) peers
  if any ((== ident) . show . pUUID) peers then
    return mUUID
  else case headMaybe filterPeer of
    Nothing -> return mUUID
    Just p -> return . return $ pUUID p
    

requestFrom' :: (HasMailSyncConfig env) => String -> RIO env (Maybe String)
requestFrom' ident = runMaybeT $ do
  uuid <- MaybeT $ getPeerUUID ident
  gs <- lift slurpGlobalState
  return $ "peerToPeerProtocol sendSince " ++ show (myUUID gs) ++ " " ++ show (peerSince uuid gs)

requestFrom :: (HasMailSyncConfig env) => String -> RIO env ()
requestFrom ident = do
 mUUID <- getPeerUUID ident 
 case mUUID of
   Nothing -> return ()
   Just peer -> printSomethingOfConfig $ \gs ->
      "peerToPeerProtocol sendSince " ++ show (myUUID gs) ++ " " ++ show (peerSince peer gs)

encodeGlobalState :: RIO env ()
encodeGlobalState = do
  (mGs :: Maybe GlobalState) <- readText <$> B.getContents
  maybe (return ())`flip` mGs $ LB.hPut stdout . encode
