module Init 
     ( init
     ) where

import           RIO
import           RIO.FilePath ((</>))
import           RIO.Directory (doesFileExist)
import qualified RIO.Text as T

import qualified Data.UUID.V4 as UUID

import Data
import IOClasses
import IOData (writeGlobalState_)
import Lock.Sync
import ExitCodes
import Utils (printErrorMessage)

init :: (HasLogFunc env, HasMailSyncConfig env) => String -> RIO env ()
init myName = do
  syncDir <- view mailSyncDirL
  withWriteLock $ do
    exists  <- doesFileExist $ syncDir </> globalStateFileName
    if exists then failAlreadyInitialized else doInit myName
   
failAlreadyInitialized :: RIO env ()
failAlreadyInitialized = do
  printErrorMessage "Tried to initialize, but global state already exists"
  exitWith $ ExitFailure exitAlreadyInit

doInit :: (HasLogFunc env, HasMailSyncConfig env) => String -> RIO env ()
doInit myName = do
  gs <- emptyGlobalState myName <$> liftIO UUID.nextRandom
  writeGlobalState_ gs
  logInfo . display $ "Empty init successfull, globalState: " `T.append` (T.pack $ show gs)
