module Lib
    ( innerMain
    ) where

import GHC.IO.Encoding (setFileSystemEncoding, char8)
import RIO

import Apply (applySingleCli, applyManyBinary)
import Cli
import CliUtils
import Data
import Init
import InitialSync (initialSyncForPeer, initialSyncFrom)
import IOClasses
import IOData
import Purge (purge, dropPeer)
import Send
import Sync
import Update (update, updateAll)
import Uux (scheduleUux)

innerMain :: IO ()
innerMain = run $ do
  liftIO $ setFileSystemEncoding char8
  options <- view cliOptionsL
  case optCommand options of
    Initialize myName -> init myName
    SyncOne mb -> syncMailBox mb
    ApplyOne peer -> applySingleCli peer
    ApplyMany peer -> applyManyBinary peer
    SendSince peer since -> sendSince peer $ Id { fromId = since }
    Purge -> purge
    PrintUUID -> printMyUUID
    ShowGlobalState -> showGlobalState
    EncodeGlobalState -> encodeGlobalState
    RequestFrom peer -> requestFrom peer
    InitialSyncForPeer peer name -> initialSyncForPeer peer name
    InitialSyncFrom -> initialSyncFrom
    SyncAllMailBoxes -> syncAllMailBoxes
    SyncAllMailBoxesInit -> syncAllMailBoxesInit
    Update peer -> update peer
    UpdateAll -> updateAll
    DropPeer peer -> dropPeer peer
    Uux peer cmd args -> scheduleUux peer (cmd, args)
