{-# LANGUAGE ForeignFunctionInterface
           , CPP
	   #-}
module Lock
  (  withLock
  , lock
  , unlock
  , SharedExclusive(Shared, Exclusive)
  , Block (Block, NoBlock)
  , Lock
  )
where

import RIO

import Data.Bits                ((.|.))
import Foreign.C.Types             (CInt(..))
import System.Posix.Error          ( throwErrnoPathIfMinus1Retry_ )
import System.Posix.IO             ( openFd
                                   , defaultFileFlags
                                   , closeFd
                                   , OpenMode(ReadOnly)
                                   )
import System.Posix.Types          ( Fd(Fd) )


#include <sys/file.h>


c_LOCK_SH, c_LOCK_EX, c_LOCK_UN, c_LOCK_NB :: CInt
c_LOCK_SH = #const LOCK_SH
c_LOCK_EX = #const LOCK_EX
c_LOCK_UN = #const LOCK_UN
c_LOCK_NB = #const LOCK_NB

foreign import ccall safe "sys/file.h flock" flock :: CInt -> CInt -> IO CInt

data SharedExclusive = Shared | Exclusive

data Block = Block | NoBlock

newtype Lock = Lock CInt

withLock :: (MonadUnliftIO m) => FilePath -> SharedExclusive -> Block -> m a -> m a
withLock fp se b x =
  bracket
    (lock fp se b)
    unlock
    (const x)

operation :: SharedExclusive -> Block -> CInt
operation se b =
  case b of
    Block   -> op
    NoBlock -> op .|. c_LOCK_NB
  where
    op = case se of
           Shared    -> c_LOCK_SH
           Exclusive -> c_LOCK_EX


lock :: MonadIO m => FilePath -> SharedExclusive -> Block -> m Lock
lock fp se b = liftIO $
  do Fd fd <- openFd fp ReadOnly Nothing defaultFileFlags
     throwErrnoPathIfMinus1Retry_ "flock" fp $ flock fd (operation se b)
     return (Lock fd)

unlock :: MonadIO m => Lock -> m ()
unlock (Lock fd) = liftIO $
  do _ <- flock fd c_LOCK_UN
     closeFd (Fd fd)
