module Match ( getMailBoxes
             , doesMailBoxExist
             , mailBoxGuard
             , mailBoxGuard_
             , mailBoxIf
             ) where

import RIO
import RIO.FilePath ((</>))
import RIO.Directory (listDirectory, doesDirectoryExist)
import Text.Regex.PCRE

import Match.Predicate
import IOClasses

workPredicate :: FilePath -> Predicate -> Bool -> Bool
workPredicate fp (Include s) p = p || fp =~ s
workPredicate fp (Exclude s) p = p && (not $ fp =~ s)

workPredicates :: FilePath -> [Predicate] -> Bool
workPredicates fp = foldl' (workPredicate fp `flip`) False

filterMailBoxesPredicates :: [Predicate] -> [FilePath] -> [FilePath]
filterMailBoxesPredicates ps = filter (workPredicates `flip` ps)

getMailBoxes :: (HasMailRoot env, HasMailSyncConfig env )
             => RIO env [FilePath]
getMailBoxes = do
  mailRoot <- view mailDirRootL
  ps <- view mailSyncPredicatesL
  contentOfMailRoot <- listDirectory mailRoot
  mb <- filterM (\x -> doesDirectoryExist $ mailRoot </> x) contentOfMailRoot
  return $ filterMailBoxesPredicates ps mb

doesMailBoxExist :: (HasMailSyncConfig env)
                 => FilePath
                 -> RIO env Bool
doesMailBoxExist fp = view mailSyncPredicatesL >>= return . workPredicates fp

mailBoxGuard :: (HasMailSyncConfig env)
             => a
             -> FilePath
             -> RIO env a
             -> RIO env a
mailBoxGuard a fp it = do
  b <- doesMailBoxExist fp
  if b then it else return a

mailBoxIf :: (HasMailSyncConfig env)
          => FilePath
          -> RIO env a
          -> RIO env a
          -> RIO env a
mailBoxIf fp yes no = do
  b <- doesMailBoxExist fp
  if b then yes else no

mailBoxGuard_ :: (HasMailSyncConfig env)
              => FilePath
              -> RIO env ()
              -> RIO env ()
mailBoxGuard_ = mailBoxGuard ()
