module Send ( sendSince
            )
where 

import RIO
import qualified RIO.ByteString.Lazy as LB
import qualified RIO.Set as S

import IOClasses
import IOData
import Data.Binary (encode)
import Data
import RemoteUpdates (acceptPeer, peerKnown)

isForPeer :: PeerId -> Change -> Bool
-- In case of an execution request, the change is only relevant for the
-- designated receipient; otherwise we flod the tree, i.e., forward to all peers
-- except the one we received the request from.
isForPeer peer (Change {changePatch = ChangeUUX (UUX {uuxPeer = rcptPeer})}) = peer == rcptPeer
isForPeer peer change = peer /= (changeSource change)

filterForPeer :: PeerId -> PeerState -> Change -> Bool
filterForPeer peer since = liftA2 (&&) (isForPeer peer) ((since <=) . changeId)

sendSince :: (HasMailSyncConfig env)
          => PeerId -> PeerState -> RIO env ()
sendSince peer since = do
  writePeerState peer since
  gs <- slurpGlobalState
  let toSend = S.filter (filterForPeer peer since) $ history gs
  mapM_ (LB.hPut stdout) . S.toAscList $ S.map encode toSend
  when (not $ peerKnown gs peer) $ do
    withGlobalStateLocked $ \ gs' -> do
     writeGlobalState_ $ acceptPeer gs' peer
