module Utils ( hasAnyP
             , maybeReadBinaryFile
             , maybeDoIO
             , exceptNothing
             , (>>=?)
             , (>>=??)
             , rightToMaybe
             , maybeToRight
             , mapBoth
             , printErrorMessage
             )
where

import RIO
import qualified RIO.ByteString as B
import qualified RIO.Text as T
import System.IO.Error (IOError, isDoesNotExistError)

hasAnyP :: a -> [a->Bool] -> Bool
hasAnyP a ps = or $ map ($ a) ps

maybeDoIO :: MonadUnliftIO m => m a -> [IOError -> Bool] -> m (Maybe a)
maybeDoIO a ps = (return <$> a) `catch` \e ->
  if e `hasAnyP` ps then return Nothing else throwIO e

maybeReadBinaryFile :: MonadUnliftIO m => FilePath -> m (Maybe ByteString)
maybeReadBinaryFile fp = readFileBinary fp `maybeDoIO` [isDoesNotExistError]

exceptNothing :: MonadUnliftIO m => m (Maybe a) -> m (Maybe a)
exceptNothing = handleAny . const $ return Nothing

sJoin :: (Monad m, Traversable m, Applicative f) => (a -> f(m b)) -> m a -> f(m b)
sJoin = ((fmap join . sequenceA) .) . fmap

(>>=?) :: (Monad m, Monad m', Traversable m') => m (m' a) -> (a -> m (m' b)) -> m (m' b)
(>>=?) a f = a >>= sJoin f 

(>>=??) :: (Monad m, Monad m', Traversable m') => m (m' a) -> (a -> m ()) -> m (m' ())
(>>=??) a f = a >>=? (fmap return . f)

rightToMaybe :: Either a b -> Maybe b
rightToMaybe = either (const Nothing) Just

maybeToRight :: a -> Maybe b -> Either a b
maybeToRight a = maybe (Left a) Right

mapBoth :: (a -> c) -> (b -> d) -> Either a b -> Either c d
mapBoth f _ (Left x) = Left (f x)
mapBoth _ f (Right x) = Right (f x)

printErrorMessage :: String -> RIO env ()
printErrorMessage = (B.hPutStr stderr) . encodeUtf8 . T.pack . ( ++ "\n")
