module Uux ( scheduleUux
           , applyUux
           )
where

import RIO
import RIO.Process

import qualified RIO.ByteString as BS
import qualified RIO.Text as T

import Data (PeerId)
import IOClasses
import IOData (withGlobalStateLocked, writeGlobalState_)
import LocalUpdates (addUux)

scheduleUux :: (HasMailSyncConfig env)
            => PeerId
            -> (String, [String])
            -> RIO env ()
scheduleUux peer cmd = withGlobalStateLocked $ \gs ->
  BS.hGetContents stdin >>=
    writeGlobalState_ . addUux gs peer cmd

applyUux :: ( HasMailSyncConfig env
            , HasLogFunc env
            , HasProcessContext env
            )
         => PeerId
         -> (String, [String])
         -> ByteString
         -> RIO env ()
applyUux peer cmd@(prg, args) input = do
  acceptableCmds <- view mailSyncAcceptableCmdsL
  if not $ prg `elem` acceptableCmds
     then logError . display . T.pack $ "Refusing to execute: " ++ show cmd ++ " (with input " ++ show input ++ ") for " ++ show peer
     else do
       (exit, out, err) <- proc prg args $ readProcess . setStdin (byteStringInput $ fromStrictBytes input)
       case exit of
         ExitSuccess -> return ()
         _ -> logError . display . T.pack
                $ "Error while executing " ++ show cmd ++ "(with input " ++ show input ++ ") for " ++ show peer
                ++ ": " ++ show (exit, out, err)
