#!/bin/env bats

load '../../bats-support/load'
load '../../bats-file/load'

function setup {
    STACK_INSTALL_ROOT=$(cd mail-sync && stack path --local-install-root)
    MAIL_SYNC="${STACK_INSTALL_ROOT}/bin/mail-sync"
    TEST_TMP_DIR="$(temp_make)"
    MAIL_SYNC_1="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config"
    MAIL_SYNC_2="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config2"
    mkdir "${TEST_TMP_DIR}/sync"
    mkdir "${TEST_TMP_DIR}/sync2"
    mkdir "${TEST_TMP_DIR}/mail"
    cp test/data/test_config "${TEST_TMP_DIR}"
    cp test/data/test_config "${TEST_TMP_DIR}/test_config2"
    ed "${TEST_TMP_DIR}/test_config" << EOF
s|ROOT|${TEST_TMP_DIR}/mail
s|SYNC|${TEST_TMP_DIR}/sync
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config"])
w
q
EOF
    ed "${TEST_TMP_DIR}/test_config2" << EOF
s|ROOT|${TEST_TMP_DIR}/mail2
s|SYNC|${TEST_TMP_DIR}/sync2
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config2"])
w
q
EOF
    cp -r test/data/testMails "${TEST_TMP_DIR}/mail"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/new"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/tmp"
}

function teardown {
    temp_del "$TEST_TMP_DIR"
}

function new_mail_at_1 {
   cat > "${TEST_TMP_DIR}/mail/testMails/new/"`date +%s`.$$_1.`hostname` <<'EOI'
From: me@example.com
To: you@example.org
Subject: new mail
Message-Id: <123@example.com>

Hi,
this is a test
EOI
  ls -R "${TEST_TMP_DIR}/mail"
}

function new_mail_at_2 {
   cat > "${TEST_TMP_DIR}/mail2/testMails/new/"`date +%s`.$$_1.`hostname` <<'EOI'
From: me@example.com
To: you@example.org
Subject: new mail
Message-Id: <345@example.com>

Hi,
this is a test
EOI
  ls -R "${TEST_TMP_DIR}/mail2"
}

@test "simpleInitialSync" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  run sh -c "${MAIL_SYNC_2} setup init test2"

  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 0 ]

  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]

  # SInce nothing changed on peer 2, they have nothing to tell to peer 1
  REQUEST=`${MAIL_SYNC_1} peerToPeerProtocol requestFrom ${UUID2}`
  run sh -c "${MAIL_SYNC_2} ${REQUEST} > ${TEST_TMP_DIR}/emptyAnswer"
  [ "${status}" -eq 0 ]
  [ -f "${TEST_TMP_DIR}/emptyAnswer" ]
  [ '!' -s "${TEST_TMP_DIR}/emptyAnswer" ]
}

@test "initialHandshake" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 0 ]
  # Repearting the inital sync of all mailboxes is OK
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 0 ]
  

  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  # Since #1 has a peer now, it should refuse to discard any new observations
  new_mail_at_1
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  run sh -c "${MAIL_SYNC_1} syncAllMailBoxes"
  ${MAIL_SYNC_1} debugState showGlobalState
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} purge"
  [ "$status" -eq 0 ]
  
  # Peer #2 can accept the initial bundle
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 1 ]
  # ... and syncing later should get the new message
  REQUEST=`${MAIL_SYNC_2} peerToPeerProtocol requestFrom test1`
  run sh -c "${MAIL_SYNC_1} ${REQUEST} > ${TEST_TMP_DIR}/update"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_2} peerToPeerProtocol applyMany test1  < ${TEST_TMP_DIR}/update"
  [ "$status" -eq 0 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 2 ]

  # local changes on peer 2
  mailname=$(cd ${TEST_TMP_DIR}/mail2/testMails/new && ls 1*)
  mv ${TEST_TMP_DIR}/mail2/testMails/new/$mailname ${TEST_TMP_DIR}/mail2/testMails/cur/$mailname:2,S 
  run sh -c "${MAIL_SYNC_2} syncAllMailBoxes"
  [ ${status} -eq 0 ]
  REQUEST_1=`${MAIL_SYNC_1} peerToPeerProtocol requestFrom test2`
  run sh -c "${MAIL_SYNC_2} ${REQUEST_1} > ${TEST_TMP_DIR}/update2"
  [ ${status} -eq 0 ]
  ls -al ${TEST_TMP_DIR}/update2
  run sh -c "${MAIL_SYNC_1} peerToPeerProtocol applyMany test2  < ${TEST_TMP_DIR}/update2"
  run sh -c " ls -l ${TEST_TMP_DIR}/mail/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 14 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 1 ]

}

@test "updateFromServer" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  ed "${TEST_TMP_DIR}/test_config" << EOF
s|Nothing|Just ["${MAIL_SYNC}", "-c", "${TEST_TMP_DIR}/test_config2"]|
w
q
EOF
  run sh -c "${MAIL_SYNC_1} syncAllMailBoxes"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} update test2"
  [ "$status" -eq 0 ]
  ls -lR "${TEST_TMP_DIR}/mail2/"
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 1 ]
  new_mail_at_2
  run sh -c "${MAIL_SYNC_2} syncAllMailBoxes"
  run sh -c "${MAIL_SYNC_1} update test2"
  [ "$status" -eq 0 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 2 ]
}

@test "updateFromClient" {
  mv "${TEST_TMP_DIR}/mail" "${TEST_TMP_DIR}/mail.use_later" 
  run sh -c "${MAIL_SYNC_1} setup init test1"
  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID1=`${MAIL_SYNC_1} setup printUUID`
  run sh -c "${MAIL_SYNC_2} setup initialSyncFor ${UUID1} test1 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  ed "${TEST_TMP_DIR}/test_config" << EOF
s|Nothing|Just ["${MAIL_SYNC}", "-c", "${TEST_TMP_DIR}/test_config2"]|
w
q
EOF
  rm -rf "${TEST_TMP_DIR}/mail"
  mv "${TEST_TMP_DIR}/mail.use_later" "${TEST_TMP_DIR}/mail"
  run sh -c "${MAIL_SYNC_1} syncAllMailBoxes"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} update test2"
  [ "$status" -eq 0 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 1 ]
  new_mail_at_2
  run sh -c "${MAIL_SYNC_2} syncAllMailBoxes"
  run sh -c "${MAIL_SYNC_1} update test2"
  [ "$status" -eq 0 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 2 ]
}

@test "updateRecipientInitiated" {
  mv "${TEST_TMP_DIR}/mail" "${TEST_TMP_DIR}/mail.use_later" 
  run sh -c "${MAIL_SYNC_1} setup init test1"
  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID1=`${MAIL_SYNC_1} setup printUUID`
  run sh -c "${MAIL_SYNC_2} setup initialSyncFor ${UUID1} test1 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  ed "${TEST_TMP_DIR}/test_config2" << EOF
s|Nothing|Just ["${MAIL_SYNC}", "-c", "${TEST_TMP_DIR}/test_config"]|
w
q
EOF
  rm -rf "${TEST_TMP_DIR}/mail"
  mv "${TEST_TMP_DIR}/mail.use_later" "${TEST_TMP_DIR}/mail"
  run sh -c "${MAIL_SYNC_1} syncAllMailBoxes"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_2} update test1"
  [ "$status" -eq 0 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/cur/ | wc -l"
  [ "${lines[0]}" -eq 13 ]
  run sh -c " ls -l ${TEST_TMP_DIR}/mail2/testMails/new/ | wc -l"
  [ "${lines[0]}" -eq 1 ]
}
