#!/bin/env bats

load '../../bats-support/load'
load '../../bats-file/load'

function setup {
    STACK_INSTALL_ROOT=$(cd mail-sync && stack path --local-install-root)
    MAIL_SYNC="${STACK_INSTALL_ROOT}/bin/mail-sync"
    TEST_TMP_DIR="$(temp_make)"
    MAIL_SYNC_1="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config"
    MAIL_SYNC_2="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config2"
    mkdir "${TEST_TMP_DIR}/sync"
    mkdir "${TEST_TMP_DIR}/sync2"
    mkdir "${TEST_TMP_DIR}/mail"
    cp test/data/test_config "${TEST_TMP_DIR}"
    cp test/data/test_config "${TEST_TMP_DIR}/test_config2"
    ed "${TEST_TMP_DIR}/test_config" << EOF
s|ROOT|${TEST_TMP_DIR}/mail
s|SYNC|${TEST_TMP_DIR}/sync
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config"])
w
q
EOF
    ed "${TEST_TMP_DIR}/test_config2" << EOF
s|ROOT|${TEST_TMP_DIR}/mail2
s|SYNC|${TEST_TMP_DIR}/sync2
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config2"])
w
q
EOF
    cp -r test/data/testMails "${TEST_TMP_DIR}/mail"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/new"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/tmp"
}

function teardown {
    temp_del "$TEST_TMP_DIR"
}

@test "syntaxError" {
  run sh -c "${MAIL_SYNC_1} --help"
  echo "$output"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} thisIsNotASupportedCommand"
  echo "$output"
  [ "$status" -eq 97 ]
}

@test "alreadyInit" {
  run sh -c "${MAIL_SYNC_1} setup init"
  [ "$status" -eq 97 ]
  echo "$output"
  run sh -c "${MAIL_SYNC_1} setup init mailsync1"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup init mailsync1"
  [ "$status" -eq 66 ]
  echo "$output"
}

@test "syncAllMailboxesInitial" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  echo "$output"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor cce88c2f-4e6e-48c1-a100-c32afec78072 whoever > /dev/null"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 67 ]
}

@test "initialSyncFrom" {
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom"
  echo "$output"
  [ "$status" -eq 65 ]
  run sh -c "${MAIL_SYNC_2} setup init test2"
  echo "$output"
  [ "$status" -eq 0 ]
  mkdir -p "${TEST_TMP_DIR}/mail2"
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom </dev/null"
  echo "$output"
  [ "$status" -eq 68 ]
  rm -rf "${TEST_TMP_DIR}/mail2"
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom </dev/null"
  echo "$output"
  [ "$status" -eq 33 ]

  run sh -c "${MAIL_SYNC_1} setup init test1"
  [ "$status" -eq 0 ]
  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  echo "$output"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  echo "$output"
  [ "$status" -eq 67 ]
}

@test "initialSyncFor" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor cce88c2f-4e6e-48c1-a100-c32afec78072 test2 > /dev/null"
  echo "$output"
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor cce88c2f-4e6e-48c1-a100-c32afec78072 test2 > /dev/null"
  echo "$output"
  [ "$status" -eq 69 ]
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor fafeae42-d446-41e4-9b5a-84839dcdabac test2 > /dev/null"
  echo "$output"
  [ "$status" -eq 0 ]
  echo -- "$output" | grep -i warning
}
