#!/bin/env bats

load '../../bats-support/load'
load '../../bats-file/load'

function setup {
    STACK_INSTALL_ROOT=$(cd mail-sync && stack path --local-install-root)
    MAIL_SYNC="${STACK_INSTALL_ROOT}/bin/mail-sync"
    TEST_TMP_DIR="$(temp_make)"
    MAIL_SYNC_1="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config"
    MAIL_SYNC_2="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config2"
    mkdir "${TEST_TMP_DIR}/sync"
    mkdir "${TEST_TMP_DIR}/sync2"
    mkdir "${TEST_TMP_DIR}/mail"
    cp test/data/test_config "${TEST_TMP_DIR}"
    cp test/data/test_config "${TEST_TMP_DIR}/test_config2"
    ed "${TEST_TMP_DIR}/test_config" << EOF
s|ROOT|${TEST_TMP_DIR}/mail
s|SYNC|${TEST_TMP_DIR}/sync
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config"])
w
q
EOF
    ed "${TEST_TMP_DIR}/test_config2" << EOF
s|ROOT|${TEST_TMP_DIR}/mail2
s|SYNC|${TEST_TMP_DIR}/sync2
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config2"])
w
q
EOF
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/cur"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/new"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/tmp"
    cat <<EOF > ${TEST_TMP_DIR}/test_sendmail
#!/bin/sh
echo \$* > ${TEST_TMP_DIR}/uux.save
echo >> ${TEST_TMP_DIR}/uux.save
cat >> ${TEST_TMP_DIR}/uux.save
EOF
    chmod +x ${TEST_TMP_DIR}/test_sendmail
    cat <<EOF > ${TEST_TMP_DIR}/test_uux_cmd
#!/bin/sh
echo long cmd by \$\$ >> ${TEST_TMP_DIR}/uux.save
cat >> /dev/null
EOF
    chmod +x ${TEST_TMP_DIR}/test_uux_cmd
    cat <<EOF > ${TEST_TMP_DIR}/test_long_uux_cmd
#!/bin/sh
sleep 2
echo long cmd by \$\$ >> ${TEST_TMP_DIR}/uux.save
cat >> /dev/null
EOF
    chmod +x ${TEST_TMP_DIR}/test_long_uux_cmd
}

function teardown {
    temp_del "$TEST_TMP_DIR"
}

@test "uuxBasic" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  echo $status
  echo $output
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 0 ]
  

  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  ${MAIL_SYNC_1} debugState showGlobalState
  [ "$status" -eq 0 ]
  
  # Peer #2 can accept the initial bundle
  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  run sh -c "echo Hello World | ${MAIL_SYNC_1} uux ${UUID2} -- ${TEST_TMP_DIR}/test_sendmail -f me@example.com"
  ${MAIL_SYNC_1} debugState showGlobalState
  
  ed "${TEST_TMP_DIR}/test_config" << EOF
s|Nothing|Just ["${MAIL_SYNC}", "-c", "${TEST_TMP_DIR}/test_config2"]|
w
q
EOF
  ed "${TEST_TMP_DIR}/test_config2" << EOF
s|/usr/sbin/sendmail|${TEST_TMP_DIR}/test_sendmail|
w
q
EOF
  run sh -c "${MAIL_SYNC_1} update test2"
  cat ${TEST_TMP_DIR}/sync2/log
  ${MAIL_SYNC_2} debugState showGlobalState
  cat ${TEST_TMP_DIR}/uux.save
  cmp - ${TEST_TMP_DIR}/uux.save <<'EOF'
-f me@example.com

Hello World
EOF
}

@test "uuxOutOfOrder" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  [ "$status" -eq 0 ]
  UUID1=`${MAIL_SYNC_1} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 0 ]

  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  run sh -c "echo Hello World | ${MAIL_SYNC_1} uux ${UUID2} -- ${TEST_TMP_DIR}/test_uux_cmd"
  [ "$status" -eq 0 ]

  ed "${TEST_TMP_DIR}/test_config2" << EOF
s|/usr/sbin/sendmail|${TEST_TMP_DIR}/test_uux_cmd|
w
q
EOF
  run sh -c "${MAIL_SYNC_1} peerToPeerProtocol sendSince ${UUID2} 0 > ${TEST_TMP_DIR}/from_1"
  [ "$status" -eq 0 ]
  run sh -c  "cat ${TEST_TMP_DIR}/from_1 | ${MAIL_SYNC_2} peerToPeerProtocol applyMany ${UUID1}"
  [ "$status" -eq 0 ]
  run sh -c  "cat ${TEST_TMP_DIR}/from_1 | ${MAIL_SYNC_2} peerToPeerProtocol applyMany ${UUID1}"
  [ "$status" -eq 0 ]

  echo
  echo "=== log of peer 2 ==="
  cat "${TEST_TMP_DIR}/sync2/log"
  echo "=== end log of peer 2 ==="
  echo
  echo "=== uux.save ==="
  cat "${TEST_TMP_DIR}/uux.save"
  echo "=== end of uux.save ==="
  echo
  COUNT=`cat "${TEST_TMP_DIR}/uux.save" | wc -l`
  [ "${COUNT}" -eq 1 ]
}

@test "uuxRace" {
  run sh -c "${MAIL_SYNC_1} setup init test1"
  [ "$status" -eq 0 ]
  UUID1=`${MAIL_SYNC_1} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup syncAllMailBoxesInit"
  [ "$status" -eq 0 ]

  run sh -c "${MAIL_SYNC_2} setup init test2"
  UUID2=`${MAIL_SYNC_2} setup printUUID`
  run sh -c "${MAIL_SYNC_1} setup initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  run sh -c "${MAIL_SYNC_2} setup initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  run sh -c "echo Hello World | ${MAIL_SYNC_1} uux ${UUID2} -- ${TEST_TMP_DIR}/test_long_uux_cmd"
  [ "$status" -eq 0 ]

  ed "${TEST_TMP_DIR}/test_config2" << EOF
s|/usr/sbin/sendmail|${TEST_TMP_DIR}/test_long_uux_cmd|
w
q
EOF
  run sh -c "${MAIL_SYNC_1} peerToPeerProtocol sendSince ${UUID2} 0 > ${TEST_TMP_DIR}/from_1"
  [ "$status" -eq 0 ]
  inotifywait -t 5 -m ${TEST_TMP_DIR}/sync2/master.lock &
  cat "${TEST_TMP_DIR}/from_1" | ${MAIL_SYNC_2} peerToPeerProtocol applyMany "${UUID1}" &
  cat "${TEST_TMP_DIR}/from_1" | ${MAIL_SYNC_2} peerToPeerProtocol applyMany "${UUID1}" &
  lsof +r 1  ${TEST_TMP_DIR}/sync2/master.lock &
  wait

  echo
  echo "=== log of peer 2 ==="
  cat "${TEST_TMP_DIR}/sync2/log"
  echo "=== end log of peer 2 ==="
  echo
  echo "=== uux.save ==="
  cat "${TEST_TMP_DIR}/uux.save"
  echo "=== end of uux.save ==="
  echo
  COUNT=`cat "${TEST_TMP_DIR}/uux.save" | wc -l`
  [ "${COUNT}" -eq 1 ]
}
